package org.rentierdb.objectmanagement.graph.scheme;

import java.util.Collection;
import java.util.Map;

import org.rentierdb.common.types.IDataType;
import org.rentierdb.objectmanagement.graph.vertex.IVertex;

/**
 * This interface describes the structure of a graph as it exists in the Object Management.
 * @author Jan-Christoph Klie
 * @version 0.1
 */
public interface IVertexScheme {
	
	//####################Create######################
	
	/**
	 * Creates a vertex with the structured attribute specified by this vertex scheme and marks the relationship in it.
	 * @return Returns a vertex using this vertex scheme
	 */
	IVertex createVertex();	
	
	//####################Read######################
	
	/**
	 * Returns a collection containing all attribute names of this vertex scheme
	 * @return Returns a collection containing all attribute names of this vertex scheme
	 */
	Collection<IDataType<? extends Comparable<?>>> getAttributes();
	
	/**
	 * Searches the vertex scheme for an attribute with the specified name and returns if found its attribute value, else null.
	 * @param name The name of the attribute to find
	 * @return The attribute value or null on found
	 */
	IDataType<? extends Comparable<?>> getAttributeByName(String name);
	
	//####################Update######################
	
	/**
	 * Adds an attribute with the specified name and the given value to the vertex scheme
	 * @param name The name of the attribute to add
	 * @param value The value of the added attribute
	 * @param dataTypeClass The data type type
	 */
	void addAttribute(String name, Class<? extends IDataType<? extends Comparable<?>>> dataTypeClass);
	
	/**
	 * Adds a list of attributes to the vertex scheme
	 * @param attributeList The list of name-value-pairs
	 */
	void addAttributes(Map<String , IDataType<? extends Comparable<?>>> attributeList);
	
	
	//####################Delete######################
	
	/**
	 * Deletes this vertex and all connected edges and attributes.
	 */
	void delete();
	
	/**
	 * Deletes the attributes specified by name if existing.
	 * @param name
	 */
	void deleteAttribute(String name);
	
	/**
	 * Deletes the attributes specified in the list if existing. 
	 * @param attributeList
	 */
	void deleteAttributes(Collection<String> attributeList);
}
