package org.rentierdb.objectmanagement.graph.vertex;

import java.util.Collection;
import java.util.Map;

import org.rentierdb.objectmanagement.graph.scheme.IVertexScheme;

/**
 * This interface describes CRUD actions specified for IVertex instances
 * @author Jan-Christoph Klie
 * @version 0.2
 */
public interface IVertexCRUD {
	
	//####################Create######################
	
	/**
	 * Creates a new, empty vertex in the data base an returns a handle for further proceedings.
	 * @return A unique and newly created IVertex instance
	 */
	IVertex createVertex();
	
	/**
	 * Creates a vertex with the structured attribute specified by this vertex scheme and marks the relationship in it.
	 * @param The scheme used to create a new vertex
	 * @return A newly and unique vertex using this vertex scheme where all attribute values are null
	 */
	IVertex createVertex(IVertexScheme scheme);
	
	/**
	 * Creates a new vertex with the specified data associated in the data base an returns a handle for further proceedings.
	 * @param data The data that will the newly created vertex will hold
	 * @return A unique and newly created IVertex instance with the specified data
	 */
	IVertex createVertex(Map<String, Comparable<?>> data);
	
	/**
	 * Creates a vertex with the structured attribute specified by this vertex scheme and marks the relationship in it.
	 * It also stores the data specified in the data map where it first tries to add them as structured attributes.
	 * @param scheme The scheme used to create a new vertex
	 * @param data The data that will the newly created vertex will hold
	 * @return A newly and unique vertex using this vertex scheme
	 */
	IVertex createVertex(IVertexScheme scheme, Map<String, Comparable<?>> data);
	
	//####################Read######################
	
	/**
	 * Searches the vertex for an attribute with the specified name and returns if found its attribute value, else null.
	 * @param vertex The vertex we search on
	 * @param name The name of the attribute to find
	 * @return The attribute value on found or null 
	 */
	Comparable<?> getAttributeByName(IVertex vertex, String name);
	
	/**
	 * Creates a map of all key/value pairs in the vertex and returns it.
	 * @param vertex The vertex we search on
	 * @return A map consisting all key/value-pairs saved in this vertex or an empty map if no data was saved.
	 */
	Map<String, Comparable<?>> getData(IVertex vertex);
	
	/**
	 * Creates a collection of all the attribute names of this vertex and returns it.
	 * @param vertex The vertex we search on
	 * @return A collection containing all the vertex attribute names or null if the vertex has no attributes
	 */
	Collection<String> getAttributeNames(IVertex vertex);
	
	/**
	 * Creates a collection of all the attribute values of this vertex and returns it.
	 * @param vertex The vertex we search on
	 * @return A collection containing all the vertex attribute values or null if the vertex has no attributes 
	 */
	Collection<Comparable<?>> getAttributeValues(IVertex vertex);	
	
	//####################Update######################
	
	/**
	 * Adds an attribute with the given value to the given vertex or changes its value to the specified value if already existing.
	 * @param vertex The vertex we update
	 * @param name The name of the attribute
	 * @param value The value the attribute will hold
	 */
	void addAttribute( IVertex vertex, String name, Comparable<?> value);
	
	/**
	 * Adds every attribute with the given value in the list to the given vertex or changes its value to the specified value if already existing.
	 * @param vertex The vertex we update
	 * @param data The map which holds the name-value-Entries
	 */
	void addAttributes( IVertex vertex, Map<String, Comparable<?>> data);	
	
	//####################Delete######################
	
	/**
	 * Deletes the given vertex from the data base.
	 * @param vertex The vertex to delete.
	 */
	void delete(IVertex vertex);
	
	/**
	 * Deletes the attribute given by the name.
	 * @param vertex The vertex whose attribute we delete
	 * @param name The name of the attribute to delete
	 */
	void deleteAttribute(IVertex vertex, String name);
}
