package org.rentierdb.simpleobjectmanagement.main;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.rentierdb.objectmanagement.IObjectManagement;
import org.rentierdb.objectmanagement.graph.edge.IEdge;
import org.rentierdb.objectmanagement.graph.edge.ISingleEdge;
import org.rentierdb.objectmanagement.graph.scheme.IVertexScheme;
import org.rentierdb.objectmanagement.graph.vertex.IVertex;
import org.rentierdb.simpleobjectmanagement.graph.SimpleSingleEdge;
import org.rentierdb.simpleobjectmanagement.graph.SimpleVertex;
import org.rentierdb.storageservice.IStorageService;
import org.rentierdb.storageservice.definitions.IGraphElementAddDefinition;
import org.rentierdb.storageservice.definitions.IGraphElementUpdateDefinition;
import org.rentierdb.storageservice.revision.IGraphElementRevision;
import org.rentierdb.storageservice.storage.AttributeType;
import org.rentierdb.storageservice.storage.IStorageAttribute;
import org.rentierdb.storageservice.transaction.ITransactionToken;


/**
 * A simple implementation of an ObjectManagement for the RentierDB
 * @author Jan-Christoph Klie
 * @version 0.3
 */
public class SimpleObjectManagement implements IObjectManagement {
	
	/**
	 * This member variable stores reference on the graph used to access its methods.
	 */
	private IStorageService storageService;
	
	/**
	 * Creates an instance of the SimpleObjectManagement class with the given storage service as back end.
	 * @param storageService The storage service to use with this graph
	 */
	public SimpleObjectManagement(IStorageService storageService) {
		this.storageService = storageService;
	}
	
	//####################Create######################
	
	@Override
	public IVertex createVertex() {
		ITransactionToken token = storageService.beginTransaction();
		IGraphElementAddDefinition definition = storageService.createAddDefinition();
		
		long id = storageService.addGraphElement(token, definition);
		
		return SimpleVertex.create(id);
	}

	@Override
	public IVertex createVertex(IVertexScheme scheme) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public IVertex createVertex(Map<String, Comparable<?>> data) {
		ITransactionToken token = storageService.beginTransaction();
		IGraphElementAddDefinition definition = storageService.createAddDefinition();
		
		for( Map.Entry<String, Comparable<?>> entry : data.entrySet()) {
			definition.addUnstructuredAttribute(entry.getKey(), entry.getValue(), AttributeType.SIMPLE);			
		}
		
		long id = storageService.addGraphElement(token, definition);
		
		return SimpleVertex.create(id);
	}

	@Override
	public IVertex createVertex(IVertexScheme scheme, Map<String, Comparable<?>> data) {		
		// TODO Auto-generated method stub
		return null;
	}
	
	@Override
	public IEdge createEdge(IVertex startVertex, IVertex endVertex) {
		ITransactionToken token = storageService.beginTransaction();
		IGraphElementUpdateDefinition updateDefinition = storageService.createUpdateDefinition();
		IGraphElementAddDefinition addDefinition = storageService.createAddDefinition();
		
		String name = startVertex.getID() + "edge" + endVertex.getID();
		
		updateDefinition.addUnstructuredAttribute(name, endVertex.getID(), AttributeType.EDGE);		
		
		long id = storageService.addGraphElement(token, addDefinition);
		storageService.updateGraphElement(token, startVertex.getID(), updateDefinition);	
		
		return SimpleSingleEdge.create(id, startVertex.getID(), endVertex.getID());
	}

	@Override
	public IEdge createEdge(IVertex startVertex, IVertex endVertex, Map<String, Comparable<?>> data) {
		// TODO Auto-generated method stub
		return null;
	}
	
	//####################Read######################
	
	@Override
	public Comparable<?> getAttributeByName(IVertex vertex, String name) {
		ITransactionToken token = storageService.beginTransaction();
		IGraphElementRevision revision = storageService.getGraphElementCurrentRevision(token, vertex.getID() );
		return (Comparable<?>) revision.getUnstructuredAttribute(name).getRawObject();
	}

	@Override
	// TODO Is expensive, cast to comparable
	public Map<String, Comparable<?>> getData(IVertex vertex) {
		ITransactionToken token = storageService.beginTransaction();
		IGraphElementRevision revision = storageService.getGraphElementCurrentRevision(token, vertex.getID() );
		Map<String, Comparable<?>> result = new HashMap<String, Comparable<?>>();
		
		for( Entry<String, IStorageAttribute<?>> entry : revision.getAllUnstructuredAttributes().entrySet() ) {
			result.put(entry.getKey(), (Comparable<?>) entry.getValue().getRawObject());
		}
		
		return result;
	}

	@Override
	public Collection<String> getAttributeNames(IVertex vertex) {
		ITransactionToken token = storageService.beginTransaction();
		IGraphElementRevision revision = storageService.getGraphElementCurrentRevision(token, vertex.getID() );
		
		return new ArrayList<String>(revision.getAllUnstructuredAttributes().keySet());
	}

	@Override
	public Collection<Comparable<?>> getAttributeValues(IVertex vertex) {
		ITransactionToken token = storageService.beginTransaction();
		IGraphElementRevision revision = storageService.getGraphElementCurrentRevision(token, vertex.getID() );
				
		return new ArrayList<Comparable<?>>(revision.getAllUnstructuredAttributes().values());
	}

	
	@Override
	public IVertex getStart(IEdge edge) {
		return SimpleVertex.create(edge.getStart());
	}

	@Override
	public IVertex getEnd(ISingleEdge singleEdge) {
		return SimpleVertex.create(singleEdge.getEnd());
	}

	@Override
	public Comparable<?> getPropertyByName(IEdge edge, String name) {		
		ITransactionToken token = storageService.beginTransaction();
		IGraphElementRevision revision = storageService.getGraphElementCurrentRevision(token, edge.getID() );
		return revision.getUnstructuredAttribute(name);
	}

	@Override
	public Map<String, Comparable<?>> getProperties(IEdge edge) {
		ITransactionToken token = storageService.beginTransaction();
		IGraphElementRevision revision = storageService.getGraphElementCurrentRevision(token, edge.getID() );
		return null;
	}

	@Override
	public Collection<Comparable<?>> getPropertyNames(IEdge edge) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Collection<Comparable<?>> getPropertyValues(IEdge edge) {
		// TODO Auto-generated method stub
		return null;
	}
	
	//####################Update######################
	
	
	@Override
	public void addAttribute(IVertex vertex, String name, Comparable<?> value) {
		ITransactionToken token = storageService.beginTransaction();		
		IGraphElementUpdateDefinition definition = storageService.createUpdateDefinition();
		
		definition.addUnstructuredAttribute(name, value, AttributeType.SIMPLE);
		
		storageService.updateGraphElement(token, vertex.getID(), definition);		
	}

	@Override
	public void addAttributes(IVertex vertex, Map<String, Comparable<?>> data) {
		ITransactionToken token = storageService.beginTransaction();
		IGraphElementUpdateDefinition definition = storageService.createUpdateDefinition();
		
		for( Entry<String, Comparable<?>> e : data.entrySet()) {
			definition.addUnstructuredAttribute(e.getKey(), e.getValue(), AttributeType.SIMPLE);
		}
		
		storageService.updateGraphElement(token, vertex.getID(), definition);		
	}
	
	//####################Delete######################

	@Override
	public void delete(IVertex vertex) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void deleteAttribute(IVertex vertex, String name) {
		// TODO Auto-generated method stub
		
	}
	
}