package org.rentierdb.simplestorageservice;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang3.tuple.Pair;
import org.rentierdb.storageservice.definitions.IGraphElementUpdateDefinition;
import org.rentierdb.storageservice.exception.StorageServiceException;
import org.rentierdb.storageservice.exception.StorageServiceGraphElementException;
import org.rentierdb.storageservice.storage.AttributeType;
import org.rentierdb.storageservice.storage.IStorageAttribute;
import org.rentierdb.storageservice.storage.IStorageGraphElement;

/**
 * This is the implementation, which represents a graph element within the
 * vertex store.
 * 
 * @author Michael Schilonka
 * @version 0.1
 * @category Implementation.StorageGraphElement
 * 
 */
public class StorageGraphElement implements IStorageGraphElement {

	// #############Private Member############

	private HashMap<Long, HashMap<Integer, LinkedList<IStorageAttribute<?>>>> structuredAttributes;

	private HashMap<Long, HashMap<Integer, Integer>> structuredAttributeRevisionMap;

	private HashMap<String, Integer> unstructuredAttributeRevisionMap;

	private LinkedHashMap<String, LinkedList<IStorageAttribute<?>>> unstructuredAttributes;

	private HashMap<Flag, Boolean> dirtyFlags = initFlags();

	private int revisionCounter;

	private String comment;

	private long creationDate;

	private long modificationDate;

	// Cache values
	private int countOfStructuredAttributes;

	private HashMap<Integer, String> schemeMapper;

	// #############Enum Flags#############

	public enum Flag {
		STRATTR, // Count of StructuredAttributes
		UNSTRATTR, // Count of UnstructuredAttributes
		SCHEMEMAPPER, // SchemeMapper cache
	}

	// #############Protected Constructor#######

	protected StorageGraphElement(GraphElementAddDefinition myAddDefinition) {
		this.revisionCounter = 1;
		this.structuredAttributes = new HashMap<Long, HashMap<Integer, LinkedList<IStorageAttribute<?>>>>();
		this.unstructuredAttributes = new LinkedHashMap<String, LinkedList<IStorageAttribute<?>>>();
		this.structuredAttributeRevisionMap = new HashMap<Long, HashMap<Integer, Integer>>();
		this.unstructuredAttributeRevisionMap = new HashMap<String, Integer>();
		this.schemeMapper = new HashMap<Integer, String>();
		this.creationDate = System.currentTimeMillis();
		this.modificationDate = System.currentTimeMillis();
		fillStorageAttribute(myAddDefinition);
	}

	// ############Private Static##############

	private static HashMap<Flag, Boolean> initFlags() {
		HashMap<Flag, Boolean> result = new HashMap<Flag, Boolean>();
		result.put(Flag.STRATTR, false);
		result.put(Flag.UNSTRATTR, false);
		result.put(Flag.SCHEMEMAPPER, false);
		return result;
	}

	// #############Public Methods############

	@SuppressWarnings("unchecked")
	@Override
	public void updateGraphElement(IGraphElementUpdateDefinition myGraphElementUpdateDefinition) throws StorageServiceGraphElementException {
		GraphElementUpdateDefinition updateDef = ((GraphElementUpdateDefinition) myGraphElementUpdateDefinition);

		// meta
		this.revisionCounter++;
		this.modificationDate = System.currentTimeMillis();

		// update comment
		if (updateDef.getComment() != null)
			this.comment = updateDef.getComment();

		// update structured attributes
		if (updateDef.getToBeUpdatedStructuredAttributes().size() != 0) {
			for (Entry<Long, HashMap<Integer, Comparable<?>>> attr : updateDef.getToBeUpdatedStructuredAttributes().entrySet()) {
				for (Entry<Integer, Comparable<?>> attrVal : attr.getValue().entrySet()) {
					AttributeType temp = structuredAttributes.get(attr.getKey()).get(attrVal.getKey()).peek().getAttributeType();
					putStructuredAttributeRevision(attr.getKey(), attrVal.getKey(), new StorageAttribute(attrVal.getValue(), temp));
				}
			}
		}

		// update unstructured attributes
		if (updateDef.getToBeUpdatedUnstructuredAttributes().size() != 0) {
			for (Entry<String, Comparable<?>> attr : updateDef.getToBeUpdatedUnstructuredAttributes().entrySet()) {
				AttributeType temp = unstructuredAttributes.get(attr.getKey()).peek().getAttributeType();
				putUnstructuredAttributeRevision(attr.getKey(), new StorageAttribute(attr.getValue(), temp));
			}
		}

		// delete unstructured attributes
		if (updateDef.getToBeDeletedUnstructuredAttributes().size() != 0) {
			for (String attrName : updateDef.getToBeDeletedUnstructuredAttributes()) {
				putUnstructuredAttributeRevision(attrName, new StorageAttribute(null, AttributeType.REMOVED));
			}
		}

		// add the new unstructured attributes
		if (updateDef.getUnstructuredAttributes().size() != 0) {
			for (Entry<String, Pair<? extends Comparable<?>, AttributeType>> unstructuredAttribute : updateDef.getUnstructuredAttributes().entrySet()) {
				StorageAttribute<?> attribute = new StorageAttribute(unstructuredAttribute.getValue().getLeft(), unstructuredAttribute.getValue().getRight());
				putNewUnstructuredAttribute(unstructuredAttribute.getKey(), attribute);
			}
		}

		// add the new structured attributes
		if (updateDef.getStructuredAttributes().size() != 0) {
			HashMap<Long, HashMap<Integer, Pair<? extends Comparable<?>, AttributeType>>> temp = updateDef.getStructuredAttributes();
			for (Entry<Long, HashMap<Integer, Pair<? extends Comparable<?>, AttributeType>>> schemeMapper : temp.entrySet()) {
				for (Entry<Integer, Pair<? extends Comparable<?>, AttributeType>> attr : schemeMapper.getValue().entrySet()) {
					StorageAttribute<?> attribute = new StorageAttribute(attr.getValue().getLeft(), attr.getValue().getRight());
					putNewStructuredAttribute(schemeMapper.getKey(), attr.getKey(), attribute);
				}
			}
		}

	}

	public Object clone() throws CloneNotSupportedException {
		return super.clone();
	}

	/*
	 * (non-Javadoc) I
	 * 
	 * @see
	 * org.rentierdb.storageservice.storage.IStorageGraphElement#getRevision()
	 */
	@Override
	public int getRevision() {
		return revisionCounter;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.rentierdb.storageservice.storage.IStorageGraphElement#getCreationDate
	 * ()
	 */
	@Override
	public long getCreationDate() {
		return creationDate;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.rentierdb.storageservice.storage.IStorageGraphElement#getModificatonDate
	 * ()
	 */
	@Override
	public long getModificatonDate() {
		return modificationDate;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.rentierdb.storageservice.storage.IStorageGraphElement#getComment()
	 */
	@Override
	public String getComment() {
		return comment;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getCountOfStructuredAttributes()
	 */
	@Override
	public int getCountOfStructuredAttributes() {
		if (!dirtyFlags.get(Flag.STRATTR)) {
			int returnvalue = 0;
			for (Map.Entry<Long, HashMap<Integer, LinkedList<IStorageAttribute<?>>>> attributeScheme : structuredAttributes.entrySet()) {
				returnvalue += attributeScheme.getValue().size();
			}
			countOfStructuredAttributes = returnvalue;
			dirtyFlags.put(Flag.STRATTR, true);
			return returnvalue;
		} else {
			return countOfStructuredAttributes;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getCountOfVertexSchemes()
	 */
	@Override
	public int getCountOfVertexSchemes() {
		return structuredAttributes.size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.rentierdb.storageservice.storage.IStorageGraphElement#getVertexSchemes
	 * ()
	 */
	@Override
	public Collection<Long> getVertexSchemes() {
		return structuredAttributes.keySet();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * derivesFromVertexScheme(long)
	 */
	@Override
	public boolean derivesFromVertexScheme(long myGraphElementID) {
		return structuredAttributes.containsKey(myGraphElementID);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getAllStructuredAttributesByGraphElementIDCurrentRevision(long)
	 */
	@Override
	public Map<Integer, IStorageAttribute<?>> getAllStructuredAttributesByGraphElementIDCurrentRevision(long myGraphElementID)
			throws StorageServiceGraphElementException {
		checkStructuredAttributesContainsKey(myGraphElementID);
		if (!structuredAttributes.isEmpty()) {
			Map<Integer, IStorageAttribute<?>> result = new HashMap<Integer, IStorageAttribute<?>>();
			HashMap<Integer, LinkedList<IStorageAttribute<?>>> temp = structuredAttributes.get(myGraphElementID);
			for (Integer schemeMapper : temp.keySet()) {
				result.put(schemeMapper, temp.get(schemeMapper).getLast());
			}
			return result;
		} else
			return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getAllStructuredAttributesByGraphElementID(long, int)
	 */
	@Override
	public Map<Integer, IStorageAttribute<?>> getAllStructuredAttributesByGraphElementID(long myGraphElementID, int myRevision)
			throws StorageServiceGraphElementException {
		checkStructuredAttributesContainsKey(myGraphElementID);
		if (!structuredAttributes.isEmpty()) {
			Map<Integer, IStorageAttribute<?>> result = new HashMap<Integer, IStorageAttribute<?>>();
			HashMap<Integer, LinkedList<IStorageAttribute<?>>> temp = structuredAttributes.get(myGraphElementID);
			for (Integer schemeMapper : temp.keySet()) {
				if (checkStructuredAttributeAgainstRevisionMap(myGraphElementID, schemeMapper, myRevision)) {
					if (temp.get(schemeMapper).size() <= myRevision - 1)
						result.put(schemeMapper, temp.get(schemeMapper).getLast());
					else
						result.put(schemeMapper, temp.get(schemeMapper).get(myRevision - 1));
				}
			}
			return result;
		} else
			return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getAllStructuredAttributesCurrentRevision()
	 */
	@Override
	public Map<Long, Map<Integer, IStorageAttribute<?>>> getAllStructuredAttributesCurrentRevision() {
		if (!structuredAttributes.isEmpty()) {
			HashMap<Long, Map<Integer, IStorageAttribute<?>>> result = new HashMap<Long, Map<Integer, IStorageAttribute<?>>>();
			for (Long graphElementId : structuredAttributes.keySet()) {
				result.put(graphElementId, getAllStructuredAttributesByGraphElementIDCurrentRevision(graphElementId));
			}
			return result;
		} else
			return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getAllStructuredAttributes(int myRevision)
	 */
	@Override
	public Map<Long, Map<Integer, IStorageAttribute<?>>> getAllStructuredAttributes(int myRevision) {
		if (!structuredAttributes.isEmpty()) {
			HashMap<Long, Map<Integer, IStorageAttribute<?>>> result = new HashMap<Long, Map<Integer, IStorageAttribute<?>>>();
			for (Long graphElementId : structuredAttributes.keySet()) {
				result.put(graphElementId, getAllStructuredAttributesByGraphElementID(graphElementId, myRevision));
			}
			return result;
		} else
			return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getAllStructuredAttributesFirstRevision()
	 */
	@Override
	public Map<Long, Map<Integer, IStorageAttribute<?>>> getAllStructuredAttributesFirstRevision() {
		if (!structuredAttributes.isEmpty()) {
			HashMap<Long, Map<Integer, IStorageAttribute<?>>> result = new HashMap<Long, Map<Integer, IStorageAttribute<?>>>();
			for (Long graphElementId : structuredAttributes.keySet()) {
				result.put(graphElementId, getAllStructuredAttributesByGraphElementID(graphElementId, 1));
			}
			return result;
		} else
			return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getStructuredAttributeCurrentRevision(long, int)
	 */
	@Override
	public IStorageAttribute<?> getStructuredAttributeCurrentRevision(long myGraphElementID, int mySchemeMapper) throws StorageServiceGraphElementException {
		checkStructuredAttributesContainsKey(myGraphElementID);
		checkStructuredAttributeContainsSchemeMapper(myGraphElementID, mySchemeMapper);
		return structuredAttributes.get(myGraphElementID).get(mySchemeMapper).getLast();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getStructuredAttributeRevision(long, int, int)
	 */
	@Override
	public IStorageAttribute<?> getStructuredAttributeRevision(long myGraphElementID, int mySchemeMapper, int myRevision)
			throws StorageServiceGraphElementException {
		checkStructuredAttributesContainsKey(myGraphElementID);
		checkStructuredAttributeContainsSchemeMapper(myGraphElementID, mySchemeMapper);
		if (checkStructuredAttributeAgainstRevisionMap(myGraphElementID, mySchemeMapper, myRevision)) {
			if (structuredAttributes.get(myGraphElementID).get(mySchemeMapper).size() <= myRevision - 1)
				return structuredAttributes.get(myGraphElementID).get(mySchemeMapper).getLast();
			else
				return structuredAttributes.get(myGraphElementID).get(mySchemeMapper).get(myRevision - 1);
		}
		throw new StorageServiceException("The StructuredAttribute of: " + myGraphElementID + " with the SchemeMapper: " + mySchemeMapper
				+ " is not available in revision" + myRevision);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getCountOfUnstructuredAttributes()
	 */
	@Override
	public int getCountOfUnstructuredAttributes() {
		return unstructuredAttributes.size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getUnstructuredAttributeCurrentRevision(java.lang.String)
	 */
	@Override
	public IStorageAttribute<?> getUnstructuredAttributeCurrentRevision(String myAttributeName) throws StorageServiceGraphElementException {
		checkUnstructuredAttributeContainsKey(myAttributeName);
		return unstructuredAttributes.get(myAttributeName).getFirst();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getUnstructuredAttributeCurrentRevision(int)
	 */
	@Override
	public IStorageAttribute<?> getUnstructuredAttributeCurrentRevision(int myAttributeMapper) throws StorageServiceGraphElementException {
		checkUnstructuredAttributeMapper(myAttributeMapper);
		return unstructuredAttributes.get(getAttributeNameBySchemeMapper(myAttributeMapper)).getFirst();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getUnstructuredAttribute(int, int)
	 */
	@Override
	public IStorageAttribute<?> getUnstructuredAttribute(int myAttributeMapper, int myRevision) throws StorageServiceGraphElementException {
		checkUnstructuredAttributeMapper(myAttributeMapper);
		unstructuredAttributes.get(getAttributeNameBySchemeMapper(myAttributeMapper));
		LinkedList<IStorageAttribute<?>> temp = unstructuredAttributes.get(getAttributeNameBySchemeMapper(myAttributeMapper));
		if (checkUnstructuredAttributeAgainstRevisionMap(schemeMapper.get(myAttributeMapper), myRevision)) {
			if (temp.size() <= myRevision - 1)
				return temp.getLast();
			else
				return temp.get(myRevision - 1);
		}
		throw new StorageServiceException("The UnstructuredAttribute " + schemeMapper.get(myAttributeMapper) + " is not available in revision" + myRevision);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getUnstructuredAttribute(java.lang.String, int)
	 */
	@Override
	public IStorageAttribute<?> getUnstructuredAttribute(String myAttributeName, int myRevision) throws StorageServiceGraphElementException {
		checkUnstructuredAttributeContainsKey(myAttributeName);
		if (checkUnstructuredAttributeAgainstRevisionMap(myAttributeName, myRevision)) {
			if (unstructuredAttributes.get(myAttributeName).size() <= myRevision - 1)
				return unstructuredAttributes.get(myAttributeName).getLast();
			else
				return unstructuredAttributes.get(myAttributeName).get(myRevision - 1);
		}
		throw new StorageServiceException("The UnstructuredAttribute " + myAttributeName + " is not available in revision" + myRevision);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getCurrentRevisionNumberOfAttribute(java.lang.String)
	 */
	@Override
	public int getCurrentRevisionNumberOfAttribute(String myAttributeName) throws StorageServiceGraphElementException {
		checkUnstructuredAttributeContainsKey(myAttributeName);
		return unstructuredAttributes.get(myAttributeName).size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getCurrentRevisionNumberOfAttribute(int)
	 */
	@Override
	public int getCurrentRevisionNumberOfAttribute(int myAttributeMapper) {
		checkUnstructuredAttributeMapper(myAttributeMapper);
		return unstructuredAttributes.get(getAttributeNameBySchemeMapper(myAttributeMapper)).size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getAllNamesAndMapperOfUnstructuredAttributes()
	 */
	@Override
	public Map<Integer, String> getAllNamesAndMapperOfUnstructuredAttributes() {
		updateSchemeMapperCache();
		return schemeMapper;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getAllUnstructuredAttributesCurrentRevision()
	 */
	@Override
	public Map<String, IStorageAttribute<?>> getAllUnstructuredAttributesCurrentRevision() {
		if (unstructuredAttributes.isEmpty())
			return null;
		HashMap<String, IStorageAttribute<?>> result = new HashMap<String, IStorageAttribute<?>>();
		Iterator<Entry<String, LinkedList<IStorageAttribute<?>>>> itr = unstructuredAttributes.entrySet().iterator();
		while (itr.hasNext()) {
			Entry<String, LinkedList<IStorageAttribute<?>>> temp = itr.next();
			result.put(temp.getKey(), temp.getValue().getLast());
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getAllUnstructuredAttributes(int myRevision)
	 */
	@Override
	public Map<String, IStorageAttribute<?>> getAllUnstructuredAttributes(int myRevision) {
		if (unstructuredAttributes.isEmpty())
			return null;
		HashMap<String, IStorageAttribute<?>> result = new HashMap<String, IStorageAttribute<?>>();
		Iterator<Entry<String, LinkedList<IStorageAttribute<?>>>> itr = unstructuredAttributes.entrySet().iterator();
		while (itr.hasNext()) {
			Entry<String, LinkedList<IStorageAttribute<?>>> temp = itr.next();
			if (checkUnstructuredAttributeAgainstRevisionMap(temp.getKey(), myRevision)) {
				if (temp.getValue().size() <= myRevision - 1)
					result.put(temp.getKey(), temp.getValue().getLast());
				else
					result.put(temp.getKey(), temp.getValue().get(myRevision - 1));
			}
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getAllUnstructuredAttributesFirstRevision()
	 */
	@Override
	public Map<String, IStorageAttribute<?>> getAllUnstructuredAttributesFirstRevision() {
		if (unstructuredAttributes.isEmpty())
			return null;
		HashMap<String, IStorageAttribute<?>> result = new HashMap<String, IStorageAttribute<?>>();
		Iterator<Entry<String, LinkedList<IStorageAttribute<?>>>> itr = unstructuredAttributes.entrySet().iterator();
		while (itr.hasNext()) {
			Entry<String, LinkedList<IStorageAttribute<?>>> temp = itr.next();
			int countrevision = unstructuredAttributeRevisionMap.get(temp.getKey());
			if (countrevision == 1)
				result.put(temp.getKey(), temp.getValue().getFirst());
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getUnstructuredAttributeName(int)
	 */
	@Override
	public String getUnstructuredAttributeName(int myAttributeMapper) throws StorageServiceGraphElementException {
		checkUnstructuredAttributeMapper(myAttributeMapper);
		return getAttributeNameBySchemeMapper(myAttributeMapper);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * getUnstructuredAttributeMapper(java.lang.String)
	 */
	@Override
	public int getUnstructuredAttributeMapper(String myAttributeName) throws StorageServiceGraphElementException {
		checkUnstructuredAttributeContainsKey(myAttributeName);
		return getSchemeMapperByAttributeName(myAttributeName);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * hasUnstructuredAttribute(java.lang.String)
	 */
	@Override
	public boolean hasUnstructuredAttribute(String myAttributeName) {
		return unstructuredAttributes.containsKey(myAttributeName);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.storage.IStorageGraphElement#
	 * hasUnstructuredAttribute(int)
	 */
	@Override
	public boolean hasUnstructuredAttribute(int AttributeMapper) {
		try {
			getAttributeNameBySchemeMapper(AttributeMapper);
		} catch (Exception e) {
			return false;
		}
		return true;
	}
	
	public void rollbackRevision(int myUntilRevision){
		if(this.getRevision() <= myUntilRevision){
			
			
		}
	}

	public void removeUnstructuredAttributeRevision(long myGraphElementID, int mySchemeMapper, int myRevision) {
		if (structuredAttributes.containsKey(myGraphElementID)) {
			if (structuredAttributes.get(myGraphElementID).containsKey(mySchemeMapper)) {
				LinkedList<IStorageAttribute<?>> temp = structuredAttributes.get(myGraphElementID).get(mySchemeMapper);
				checkStructuredAttributeAgainstRevisionMap(myGraphElementID, mySchemeMapper, myRevision);
				while (temp.size() >= myRevision - 1) {
					temp.removeLast();
				}
			}
		}
	}

	public void removeUnstructuredAttributeRevision(String myAttributeName, int myRevision) {
		if(structuredAttributes.containsKey(myAttributeName)){
			checkUnstructuredAttributeAgainstRevisionMap(myAttributeName, myRevision);
			LinkedList<IStorageAttribute<?>> temp = unstructuredAttributes.get(myAttributeName);
			while (temp.size() >= myRevision - 1) {
				temp.removeLast();
			}
		}
	}

	// ################private Helper################
	/**
	 * @param myAttributeName
	 * @throws StorageServiceGraphElementException
	 */
	private void checkUnstructuredAttributeContainsKey(String myAttributeName) throws StorageServiceGraphElementException {
		if (!unstructuredAttributes.containsKey(myAttributeName))
			throw new StorageServiceGraphElementException("The UnstructuredAttribute to the given AttributeName could not be found!");
	}

	/**
	 * @param myAttributeMapper
	 * @throws StorageServiceGraphElementException
	 */
	private void checkUnstructuredAttributeMapper(int myAttributeMapper) throws StorageServiceGraphElementException {
		if (unstructuredAttributes.size() <= myAttributeMapper)
			throw new StorageServiceGraphElementException("There is no UnstructuredAttribute with the given AttributeMapper!");
	}

	private boolean checkUnstructuredAttributeAgainstRevisionMap(String myAttributeName, int myRevision) {
		if (unstructuredAttributeRevisionMap.get(myAttributeName) > myRevision)
			return false;
		return true;
	}

	private boolean checkStructuredAttributeAgainstRevisionMap(long myGraphElement, int mySchemeMapper, int myRevision) {
		if (structuredAttributeRevisionMap.get(myGraphElement).get(mySchemeMapper) > myRevision)
			return false;
		return true;
	}

	/**
	 * @param myGraphElementID
	 * @param mySchemeMapper
	 * @throws StorageServiceGraphElementException
	 */
	private void checkStructuredAttributeContainsSchemeMapper(long myGraphElementID, int mySchemeMapper) throws StorageServiceGraphElementException {
		if (!structuredAttributes.get(myGraphElementID).containsKey(mySchemeMapper))
			throw new StorageServiceGraphElementException("The given SchemeMapperID could not be found!");
	}

	/**
	 * @param myGraphElementID
	 * @throws StorageServiceGraphElementException
	 */
	private void checkStructuredAttributesContainsKey(long myGraphElementID) throws StorageServiceGraphElementException {
		if (!structuredAttributes.containsKey(myGraphElementID))
			throw new StorageServiceGraphElementException("The given GraphElementID could not be found!");
	}

	private String getAttributeNameBySchemeMapper(int mySchemeMapper) {
		if (!dirtyFlags.get(Flag.SCHEMEMAPPER)) {
			updateSchemeMapperCache();
		}
		if (schemeMapper.containsKey(mySchemeMapper))
			return schemeMapper.get(mySchemeMapper);
		else
			throw new StorageServiceGraphElementException("There is no UnstructuredAttribute with the given AttributeMapper!");
	}

	private int getSchemeMapperByAttributeName(String myAttributeName) {
		if (!dirtyFlags.get(Flag.SCHEMEMAPPER)) {
			updateSchemeMapperCache();
		}
		for (Entry<Integer, String> mapping : schemeMapper.entrySet()) {
			if (mapping.getValue().equals(myAttributeName))
				return mapping.getKey();
		}
		return -1;
	}

	private void updateSchemeMapperCache() {
		schemeMapper.clear();
		Iterator<String> itr = unstructuredAttributes.keySet().iterator();
		for (int i = 0; i < unstructuredAttributes.size(); i++) {
			schemeMapper.put(i, itr.next());
		}
	}

	private void putNewUnstructuredAttribute(String myAttributeName, IStorageAttribute<?> myStorageAttribute) {
		if (unstructuredAttributes.containsKey(myAttributeName))
			throw new StorageServiceGraphElementException("The AttributeName " + myAttributeName + " already exists in the list of unstructured attributes");
		LinkedList<IStorageAttribute<?>> revision = new LinkedList<IStorageAttribute<?>>();
		revision.push(myStorageAttribute);
		dirtyFlags.put(Flag.UNSTRATTR, false);
		unstructuredAttributes.put(myAttributeName, revision);
		unstructuredAttributeRevisionMap.put(myAttributeName, this.revisionCounter);
	}

	private void putUnstructuredAttributeRevision(String myAttributeName, IStorageAttribute<?> myStorageAttribute) {
		if (!unstructuredAttributes.containsKey(myAttributeName))
			throw new StorageServiceGraphElementException("The AttributeName " + myAttributeName + " is not in the list of unstructured attributes");
		dirtyFlags.put(Flag.UNSTRATTR, false);
		unstructuredAttributes.get(myAttributeName).addLast(myStorageAttribute);
	}

	private void putNewStructuredAttribute(long mySchemeID, int myMapper, IStorageAttribute<?> myStorageAttribute) {
		if (structuredAttributes.containsKey(mySchemeID)) {
			if (structuredAttributes.get(mySchemeID).containsKey(myMapper))
				throw new StorageServiceGraphElementException("The attribute mapper: " + myMapper + " is already in the list!");
			LinkedList<IStorageAttribute<?>> revision = new LinkedList<IStorageAttribute<?>>();
			revision.add(myStorageAttribute);
			dirtyFlags.put(Flag.SCHEMEMAPPER, false);
			dirtyFlags.put(Flag.STRATTR, false);
			structuredAttributes.get(mySchemeID).put(myMapper, revision);
			if (structuredAttributeRevisionMap.containsKey(mySchemeID)) {
				structuredAttributeRevisionMap.get(mySchemeID).put(myMapper, this.revisionCounter);
			} else {
				HashMap<Integer, Integer> revisionmap = new HashMap<Integer, Integer>();
				revisionmap.put(myMapper, this.revisionCounter);
				structuredAttributeRevisionMap.put(mySchemeID, revisionmap);
			}
		} else {
			LinkedList<IStorageAttribute<?>> revision = new LinkedList<IStorageAttribute<?>>();
			revision.push(myStorageAttribute);
			HashMap<Integer, LinkedList<IStorageAttribute<?>>> mapper = new HashMap<Integer, LinkedList<IStorageAttribute<?>>>();
			mapper.put(myMapper, revision);
			dirtyFlags.put(Flag.SCHEMEMAPPER, false);
			dirtyFlags.put(Flag.STRATTR, false);
			structuredAttributes.put(mySchemeID, mapper);
			if (structuredAttributeRevisionMap.containsKey(mySchemeID))
				throw new StorageServiceException("There is already an entry for the revision map.");
			HashMap<Integer, Integer> revisionmap = new HashMap<Integer, Integer>();
			revisionmap.put(myMapper, this.revisionCounter);
			structuredAttributeRevisionMap.put(mySchemeID, revisionmap);
		}

	}

	private void putStructuredAttributeRevision(long mySchemeID, int myMapper, IStorageAttribute<?> myStorageAttribute) {
		if (!structuredAttributes.containsKey(mySchemeID))
			throw new StorageServiceGraphElementException("The SchemeID " + mySchemeID + " is not in the list of structured attributes");
		if (!structuredAttributes.get(mySchemeID).containsKey(myMapper))
			throw new StorageServiceGraphElementException("The SchemeMapper " + myMapper + " is not in the list of SchemeMappers");
		dirtyFlags.put(Flag.SCHEMEMAPPER, false);
		dirtyFlags.put(Flag.STRATTR, false);
		structuredAttributes.get(mySchemeID).get(myMapper).addLast(myStorageAttribute);
	}

	private void putStructuredAttribute(long mySchemeID, int myMapper, IStorageAttribute<?> myStorageAttribute) {
		if (!structuredAttributes.containsKey(mySchemeID))
			throw new StorageServiceGraphElementException("The SchemeID " + mySchemeID + " is not in the list of structured attributes");
		LinkedList<IStorageAttribute<?>> revision = new LinkedList<IStorageAttribute<?>>();
		revision.push(myStorageAttribute);
		dirtyFlags.put(Flag.SCHEMEMAPPER, false);
		dirtyFlags.put(Flag.STRATTR, false);
		structuredAttributes.get(mySchemeID).put(myMapper, revision);
	}

	private void fillStorageAttribute(GraphElementAddDefinition myAddDefinition) {

		// add comment
		comment = myAddDefinition.getComment();

		// add all the unstructured attributes
		for (Entry<String, Pair<? extends Comparable<?>, AttributeType>> unstructuredAttribute : myAddDefinition.getUnstructuredAttributes().entrySet()) {
			StorageAttribute<?> attribute = new StorageAttribute(unstructuredAttribute.getValue().getLeft(), unstructuredAttribute.getValue().getRight());
			putNewUnstructuredAttribute(unstructuredAttribute.getKey(), attribute);
		}

		// add all the structured attributes
		HashMap<Long, HashMap<Integer, Pair<? extends Comparable<?>, AttributeType>>> temp = myAddDefinition.getStructuredAttributes();

		for (Entry<Long, HashMap<Integer, Pair<? extends Comparable<?>, AttributeType>>> schemeMapper : temp.entrySet()) {
			for (Entry<Integer, Pair<? extends Comparable<?>, AttributeType>> attr : schemeMapper.getValue().entrySet()) {
				StorageAttribute<?> attribute = new StorageAttribute(attr.getValue().getLeft(), attr.getValue().getRight());
				putNewStructuredAttribute(schemeMapper.getKey(), attr.getKey(), attribute);
			}
		}

	}
}