/**
 * 
 */
package org.rentierdb.storageservice;

import java.util.List;

import org.rentierdb.common.pluginable.IPluginable;
import org.rentierdb.storageservice.definitions.IGraphElementAddDefinition;
import org.rentierdb.storageservice.definitions.IGraphElementUpdateDefinition;
import org.rentierdb.storageservice.exception.StorageServiceException;
import org.rentierdb.storageservice.revision.IGraphElementRevision;
import org.rentierdb.storageservice.transaction.ITransactionManager;
import org.rentierdb.storageservice.transaction.ITransactionToken;


/**
 * @author Michael Schilonka
 * @version 0.1
 * @category StorageService
 * 
 *           This interface describes a generic StorageService for all kinds of
 *           implementations.
 */
public interface IStorageService extends ITopologyService, IStorageServiceAdmin, ITransactionManager, IPluginable {

	/**
	 * This returns the count of all vertices within the store.
	 * 
	 * @return This returns the count of all vertices within the store.
	 */
	long getCountOfVertices();

	/**
	 * This checks whether a certain graph element exist or not.
	 * 
	 * @param myTransactionToken
	 *            The transaction token The transaction token
	 * @param myGraphElementID
	 *            The ID of the interesting graph element The ID of the
	 *            interesting graph element.
	 * @return True if the graph element exists, otherwise false.
	 */
	boolean getGraphElementExists(ITransactionToken myTransactionToken, long myGraphElementID);

	/**
	 * @param myTransactionToken
	 *            The transaction token
	 * @param myGraphElementID
	 *            The ID of the interesting graph element
	 * @return The StorageGraphElement related to the given ID in the most
	 *         recent revision.
	 * @throws StorageServiceException
	 *             If any error occurs
	 */
	IGraphElementRevision getGraphElementCurrentRevision(ITransactionToken myTransactionToken, long myGraphElementID) throws StorageServiceException;

	/**
	 * @param myTransactionToken
	 *            The transaction token
	 * @param myGraphElementID
	 *            The ID of the interesting graph element
	 * @param myRevisionID
	 *            The interesting revision. The first is 0.
	 * @return The StorageGraphElement related to the given ID in the requested
	 *         revision.
	 * @throws StorageServiceException
	 *             If any error occurs
	 */
	IGraphElementRevision getGraphElementOfRevision(ITransactionToken myTransactionToken, long myGraphElementID, int myRevisionID)
			throws StorageServiceException;

	/**
	 * @param myTransactionToken
	 *            The transaction token
	 * @param myInterestingGraphElementIDs
	 * @return A set of StorageGraphElements related to the given IDs in the
	 *         most recent revision.
	 * @throws StorageServiceException
	 *             If any error occurs
	 */
	List<IGraphElementRevision> getGraphElementsCurrentRevision(ITransactionToken myTransactionToken, List<Long> myInterestingGraphElementIDs)
			throws StorageServiceException;;

	/**
	 * @param myTransactionToken
	 *            The transaction token
	 * @param myInterestingGraphElementIDs
	 * @param myRevision
	 * @return A set of StorageGraphElements related to the given IDs in the
	 *         requested revision.
	 * @throws StorageServiceException
	 *             If any error occurs
	 */
	List<IGraphElementRevision> getGraphElementsOfRevision(ITransactionToken myTransactionToken, List<Long> myInterestingGraphElementIDs, int myRevision)
			throws StorageServiceException;

	/**
	 * @param myTransactionToken
	 *            The transaction token
	 * @param myGraphElementID
	 *            The ID of the interesting graph element
	 * @throws StorageServiceException
	 *             If any error occurs
	 */
	void removeGraphElementCurrentRevision(ITransactionToken myTransactionToken, long myGraphElementID) throws StorageServiceException;

	/**
	 * @param myTransactionToken
	 *            The transaction token
	 * @param myGraphElementID
	 *            The ID of the interesting graph element
	 * @param myRevisionID
	 * @throws StorageServiceException
	 *             If any error occurs
	 */
	void removeGraphElementOfRevision(ITransactionToken myTransactionToken, long myGraphElementID, int myRevisionID) throws StorageServiceException;

	/**
	 * @param myTransactionToken
	 *            The transaction token
	 * @param myAddDefinition
	 * @return The assigned GraphElementID
	 * @throws StorageServiceException
	 *             If any error occurs
	 */
	long addGraphElement(ITransactionToken myTransactionToken, IGraphElementAddDefinition myAddDefinition) throws StorageServiceException;

	/**
	 * @param myTransactionToken
	 *            The transaction token
	 * @param myUpdateDefinition
	 * @throws StorageServiceException
	 *             If any error occurs
	 */
	void updateGraphElement(ITransactionToken myTransactionToken, long myGraphElementID, IGraphElementUpdateDefinition myUpdateDefinition)
			throws StorageServiceException;

	/**
	 * Returns an instance of IGraphElementAddDefinition for building a graph
	 * element
	 * 
	 * @return an instance of IGraphElementAddDefinition for building a graph
	 *         element
	 */
	IGraphElementAddDefinition createAddDefinition();

	/**
	 * Returns an instance of IGraphElementUpdateDefinition for updating a graph
	 * element
	 * 
	 * @return An instance of IGraphElementUpdateDefinition for updating a graph
	 *         element
	 */
	IGraphElementUpdateDefinition createUpdateDefinition();
}
