/**
 * 
 */
package org.rentierdb.storageservice.revision;

import java.util.Collection;
import java.util.Map;

import org.rentierdb.storageservice.exception.StorageServiceGraphElementException;
import org.rentierdb.storageservice.storage.IStorageAttribute;


/**
 * This interface describes the graph element representation to the object management.
 * @author Michael Schilonka
 * @version 0.1
 * @category GraphElementStorage
 */
public interface IGraphElementRevision {
	
	//#####################IStorageGraphElement wrapper###############
	
	/**
	 * The revision number of this representation.
	 * @return The revision number of this representation.
	 */
	int getRevision(); 
	
	/**
	 * The date this graph element was created.
	 * @return The date this graph element was created.
	 */
	long getCreationDate();
	
	/**
	 * The date this graph element was modified.
	 * @return The date this graph element was modified.
	 */
	long getModificatonDate();
	
	/**
	 * The comment which is attached to this graph element
	 * @return The comment which is attached to this graph element
	 */
	String getComment();	
	//#####################Revision methods#######################
	
	/**
	 * True if this representation is the most recent, otherwise false.
	 * @return True if this representation is the most recent, otherwise false.
	 */
	boolean isCurrentRevision();
	
	//#####################Structured Attributes#####################
	/**
	 * Returns the count of all structured attributes, defined by several VertexSchemes.
	 * @return Returns the count of all structured attributes.
	 */
	 int getCountOfStructuredAttributes();
		
	/**
	 * Returns the count of VertexSchemes were structured attributes derived from.
	 * @return Returns the count of derived attributes.
	 */
	 int getCountOfVertexSchemes();
		
	/**
	 * Returns the GraphElementID / GraphElementIDs from which this graph element derived from.
	 * @return Returns all VertexSchemes this Vertex derived from.
	 */
	 Collection<Long> getVertexSchemes();
		
	/**
	 * Returns true if this graph element derives from the given Vertex, otherwise false.  
	 * @param myGraphElementID This is the interesting GraphElementID.
	 * @return Returns true if this graph element derives from the given Vertex, otherwise false. 
	 */
	 boolean derivesFromVertexScheme(long myGraphElementID);
	
	/**
	 * Returns a Map of SchemeMapper and the related attribute.
	 * @param myGraphElementID This is the GraphElementID of the VertexScheme the structured attributes are derived from.
	 * @return Returns a Map of SchemeMapper and the related attribute.
	 * @throws StorageServiceGraphElementException
	 */
	 Map<Integer,IStorageAttribute<?>> getAllStructuredAttributesByGraphElementID(long myGraphElementID) throws StorageServiceGraphElementException;
	
	 /**
	 * Returns all structured attributes in the most recent revision.
	 * @return Returns all structured attributes in the most recent revision.
	 */
	 Map<Long,Map<Integer,IStorageAttribute<?>>> getAllStructuredAttributes();
	
	/**
	 * Returns the StructuredAttribute related to a VertexScheme and the corresponding SchemeMapper in the most recent revision.
	 * @param myGraphElementID This is the GraphElementID of the VertexScheme the StructuredAttributes are derived from.
	 * @param mySchemeMapper This is the SchemeMapper, which maps an structured attribute to an unstructured attribute of an VertexScheme. 
	 * @return Returns an IStorageAttribute related to the given indicators in the most recent revision.
	 * @throws StorageServiceGraphElementException
	 */
	 IStorageAttribute<?> getStructuredAttribute(long myGraphElementID, int mySchemeMapper) throws StorageServiceGraphElementException;
		
	//##################UnstructuredAttributes##################
		
	/**
	 * @ Returns the count of all unstructured attributes.
	 * @return Returns the count of all unstructured attributes.
	 */
	 int getCountOfUnstructuredAttributes();
	
	/**
	 * Returns an UnstructuredAttribute related to the AttributeName in the current revision.
	 * @param myAttributeName The name of the interesting UnstructuredAttribute.
	 * @return Returns an UnstructuredAttribute related to the AttributeName in the current revision.
	 * @throws StorageServiceGraphElementException
	 */
	 IStorageAttribute<?> getUnstructuredAttribute(String myAttributeName) throws StorageServiceGraphElementException;
	
	/**
	 * Returns an UnstructuredAttribute in the most recent revision.
	 * @param myAttributeMapper The AttributeMapper of the interesting attribute.
	 * @return Returns an UnstructuredAttribute in the current most recent revision.
	 * @throws StorageServiceGraphElementException
	 */
	 IStorageAttribute<?> getUnstructuredAttribute(int myAttributeMapper) throws StorageServiceGraphElementException;
	
	/**
	 * Returns the most recent revision number of the UnstructuredAttribute.
	 * @param myAttributeName The name of the interesting UnstructuredAttribute.
	 * @return Returns the most recent revision number of the UnstructuredAttribute.
	 */
	 int getCurrentRevisionNumberOfAttribute(String myAttributeName);
	
	/**
	 * Returns the most recent revision number of the UnstructuredAttribute.
	 * @param myAttributeMapper The AttributeMapper of the interesting UnstructuredAttribute. 
	 * @return Returns the most recent revision number of the UnstructuredAttribute.
	 */
	 int getCurrentRevisionNumberOfAttribute(int myAttributeMapper);
	
	/**
	 * Returns all names of the UnstructuredAttributes and their related AttributeMapper.
	 * @return Returns all names of the UnstructuredAttributes and their AttributeMapper.
	 */
	 Map<Integer,String> getAllNamesAndMapperOfUnstructuredAttributes();
		
	/**
	 * Returns all UnstructuredAttributes in the most recent revision.
	 * @return Returns all UnstructuredAttributes in the most recent revision.
	 */
	 Map<String,IStorageAttribute<?>> getAllUnstructuredAttributes();
	
	/**
	 * Returns the name of the UnstructuredAttribute related to the given AttributeMapper.
	 * @param myAttributeMapper The AttributeMapper of the interesting UnstructuredAttribute. 
	 * @return Returns the name of the UnstructuredAttribute related to the given AttributeMapper.
	 * @throws StorageServiceGraphElementException
	 */
	 String getUnstructuredAttributeName(int myAttributeMapper) throws StorageServiceGraphElementException;
	
	/**
	 * Returns the AttributeMapper of the UnstructuredAttribute related to the given name.
	 * @param myAttributeName The AttributeMapper of the interesting UnstructuredAttribute. 
	 * @return Returns the AttributeMapper of the UnstructuredAttribute related to the given name.
	 * @throws StorageServiceGraphElementException
	 */
	 int getUnstructuredAttributeMapper(String myAttributeName) throws StorageServiceGraphElementException;
	 
	 /**
	  * Checks whether an UnstructuredAttribute exists. 
	 * @param myAttributeName The AttributeMapper of the interesting UnstructuredAttribute. 
	 * @return True if the UnstructuredAttribute exists, otherwise false. 
	 */
	boolean hasUnstructuredAttribute(String myAttributeName);
	 
	/**
	 * Checks whether an UnstructuredAttribute exists.
	 * @param AttributeMapper The AttributeMapper of the interesting UnstructuredAttribute. 
	 * @return True if the UnstructuredAttribute exists, otherwise false. 
	 */
	boolean hasUnstructuredAttribute(int AttributeMapper);
}
